@@ -4,14 +4,14 @@
 #
 make fts2amal.c
 PATH=$PATH:/opt/mingw/bin
-OPTS='-DTHREADSAFE=1 -DBUILD_sqlite=1 -DOS_WIN=1'
+OPTS='-DTHREADSAFE=1 -DBUILD_sqlite=1 -DSQLITE_OS_WIN=1'
 CC="i386-mingw32msvc-gcc -O2 $OPTS -Itsrc"
 NM="i386-mingw32msvc-nm"
 CMD="$CC -c fts2amal.c"
 echo $CMD
 $CMD
 echo 'EXPORTS' >fts2.def
-echo 'sqlite3_extension_init' >>fts2.def
+echo 'sqlite3_fts2_init' >>fts2.def
 i386-mingw32msvc-dllwrap \
      --def fts2.def -v --export-all \
      --driver-name i386-mingw32msvc-gcc \
