@@ -93,6 +93,14 @@ set FAULTSIM(cantopen-persistent) [list      \
   -injectuninstall cantopen_injectuninstall  \
 ]
 
+set FAULTSIM(interrupt) [list                 \
+  -injectinstall   interrupt_injectinstall    \
+  -injectstart     interrupt_injectstart      \
+  -injectstop      interrupt_injectstop       \
+  -injecterrlist   {{1 interrupted} {1 interrupt}}        \
+  -injectuninstall interrupt_injectuninstall  \
+]
+
 
 
 #--------------------------------------------------------------------------
@@ -106,13 +114,21 @@ set FAULTSIM(cantopen-persistent) [list      \
 #
 #     -test             Script to execute after -body.
 #
+#     -install          Script to execute after faultsim -injectinstall
+#
+#     -uninstall        Script to execute after faultsim -uninjectinstall
+#
 proc do_faultsim_test {name args} {
   global FAULTSIM
   
-  set DEFAULT(-faults)        [array names FAULTSIM]
+  foreach n [array names FAULTSIM] {
+    if {$n != "interrupt"} {lappend DEFAULT(-faults) $n}
+  }
   set DEFAULT(-prep)          ""
   set DEFAULT(-body)          ""
   set DEFAULT(-test)          ""
+  set DEFAULT(-install)       ""
+  set DEFAULT(-uninstall)     ""
 
   fix_testname name
 
@@ -129,7 +145,9 @@ proc do_faultsim_test {name args} {
     set faultlist [concat $faultlist $flist]
   }
 
-  set testspec [list -prep $O(-prep) -body $O(-body) -test $O(-test)]
+  set testspec [list -prep $O(-prep) -body $O(-body) \
+      -test $O(-test) -install $O(-install) -uninstall $O(-uninstall)
+  ]
   foreach f [lsort -unique $faultlist] {
     eval do_one_faultsim_test "$name-$f" $FAULTSIM($f) $testspec
   }
@@ -247,6 +265,22 @@ proc cantopen_injectstop {} {
   shmfault cantopen
 }
 
+# The following procs are used as [do_one_faultsim_test] callbacks 
+# when injecting SQLITE_INTERRUPT error faults into test cases.
+#
+proc interrupt_injectinstall {} {
+}
+proc interrupt_injectuninstall {} {
+}
+proc interrupt_injectstart {iFail} {
+  set ::sqlite_interrupt_count $iFail
+}
+proc interrupt_injectstop {} {
+  set res [expr $::sqlite_interrupt_count<=0]
+  set ::sqlite_interrupt_count 0
+  set res
+}
+
 # This command is not called directly. It is used by the 
 # [faultsim_test_result] command created by [do_faultsim_test] and used
 # by -test scripts.
@@ -256,7 +290,7 @@ proc faultsim_test_result_int {args} {
   set t [list $testrc $testresult]
   set r $args
   if { ($testnfail==0 && $t != [lindex $r 0]) || [lsearch $r $t]<0 } {
-    error "nfail=$testnfail rc=$testrc result=$testresult"
+    error "nfail=$testnfail rc=$testrc result=$testresult list=$r"
   }
 }
 
@@ -294,6 +328,8 @@ proc do_one_faultsim_test {testname args} {
   set DEFAULT(-prep)            ""
   set DEFAULT(-body)            ""
   set DEFAULT(-test)            ""
+  set DEFAULT(-install)         ""
+  set DEFAULT(-uninstall)       ""
 
   array set O [array get DEFAULT]
   array set O $args
@@ -307,6 +343,7 @@ proc do_one_faultsim_test {testname args} {
   "
 
   eval $O(-injectinstall)
+  eval $O(-install)
 
   set stop 0
   for {set iFail 1} {!$stop} {incr iFail} {
@@ -338,6 +375,7 @@ proc do_one_faultsim_test {testname args} {
     if {$nfail==0} { set stop 1 }
   }
 
+  eval $O(-uninstall)
   eval $O(-injectuninstall)
 }
 
@@ -396,7 +434,7 @@ proc do_malloc_test {tn args} {
       set zRepeat "transient"
       if {$::iRepeat} {set zRepeat "persistent"}
       restore_prng_state
-      foreach file [glob -nocomplain test.db-mj*] {file delete -force $file}
+      foreach file [glob -nocomplain test.db-mj*] {forcedelete $file}
 
       do_test ${tn}.${zRepeat}.${::n} {
   
@@ -413,7 +451,7 @@ proc do_malloc_test {tn args} {
         forcedelete test2.db-journal
         forcedelete test2.db-wal
         if {[info exists ::mallocopts(-testdb)]} {
-          file copy $::mallocopts(-testdb) test.db
+          copy_file $::mallocopts(-testdb) test.db
         }
         catch { sqlite3 db test.db }
         if {[info commands db] ne ""} {
